"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScriptoriumLambda = void 0;
const server_services_core_1 = require("@fluidframework/server-services-core");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const server_services_client_1 = require("@fluidframework/server-services-client");
class ScriptoriumLambda {
    constructor(opCollection, context, providerConfig) {
        this.opCollection = opCollection;
        this.context = context;
        this.providerConfig = providerConfig;
        this.pending = new Map();
        this.current = new Map();
        this.clientFacadeRetryEnabled = (0, server_services_core_1.isRetryEnabled)(this.opCollection);
    }
    handler(message) {
        const boxcar = (0, server_services_core_1.extractBoxcar)(message);
        for (const baseMessage of boxcar.contents) {
            if (baseMessage.type === server_services_core_1.SequencedOperationType) {
                const value = baseMessage;
                // Remove traces and serialize content before writing to mongo.
                value.operation.traces = [];
                const topic = `${value.tenantId}/${value.documentId}`;
                let pendingMessages = this.pending.get(topic);
                if (!pendingMessages) {
                    pendingMessages = [];
                    this.pending.set(topic, pendingMessages);
                }
                pendingMessages.push(value);
            }
        }
        this.pendingOffset = message;
        this.sendPending();
        return undefined;
    }
    close() {
        this.pending.clear();
        this.current.clear();
        return;
    }
    sendPending() {
        // If there is work currently being sent or we have no pending work return early
        if (this.current.size > 0 || this.pending.size === 0) {
            return;
        }
        // Swap current and pending
        const temp = this.current;
        this.current = this.pending;
        this.pending = temp;
        const batchOffset = this.pendingOffset;
        const allProcessed = [];
        // Process all the batches + checkpoint
        for (const [, messages] of this.current) {
            const processP = this.processMongoCore(messages);
            allProcessed.push(processP);
        }
        Promise.all(allProcessed).then(() => {
            this.current.clear();
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this.context.checkpoint(batchOffset);
            this.sendPending();
        }, (error) => {
            server_services_telemetry_1.Lumberjack.error("An error occured in scriptorium, going to restart", {}, error);
            this.context.error(error, { restart: true });
        });
    }
    async processMongoCore(messages) {
        return this.insertOp(messages);
    }
    async insertOp(messages) {
        var _a, _b, _c, _d, _e;
        const dbOps = messages.map((message) => (Object.assign(Object.assign({}, message), { mongoTimestamp: new Date(message.operation.timestamp) })));
        const documentId = (_b = (_a = messages[0]) === null || _a === void 0 ? void 0 : _a.documentId) !== null && _b !== void 0 ? _b : "";
        const tenantId = (_d = (_c = messages[0]) === null || _c === void 0 ? void 0 : _c.tenantId) !== null && _d !== void 0 ? _d : "";
        const sequenceNumbers = messages.map((message) => message.operation.sequenceNumber);
        const sequenceNumberRanges = (0, server_services_client_1.convertSortedNumberArrayToRanges)(sequenceNumbers);
        return (0, server_services_core_1.runWithRetry)(async () => this.opCollection.insertMany(dbOps, false), "insertOpScriptorium", 3 /* maxRetries */, 1000 /* retryAfterMs */, Object.assign(Object.assign({}, (0, server_services_telemetry_1.getLumberBaseProperties)(documentId, tenantId)), { sequenceNumberRanges }), (error) => error.code === 11000, (error) => !this.clientFacadeRetryEnabled /* shouldRetry */, undefined /* calculateIntervalMs */, undefined /* onErrorFn */, (_e = this.providerConfig) === null || _e === void 0 ? void 0 : _e.enableRunWithRetryMetricTelemetry);
    }
}
exports.ScriptoriumLambda = ScriptoriumLambda;
//# sourceMappingURL=lambda.js.map